/** FuzzyTreeParameters.java
 *  Copyright (c) 2001 Donna Malayeri
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package edu.umd.cs.fuzzyTree;

/** Parameters used by FuzzyTreeModel in its score adjusting algorithms.
 *
 *  @author     Donna Malayeri
 *  @version    24-Jul-01
 */

public class FuzzyTreeParameters
{
	/** Minimum percentage score increase when expanding */
	public float   minExpandPercent;

	/** Maximum percentage score increase when expanding */
	public float   maxExpandPercent;

	/** Minimum percentage score increase when expanding */
	public float   maxCollapsePercent;

	/** Minimum percentage score decrease when collapsing */
	public float   minCollapsePercent;

	/** Percentage by which score increase percentage is reduced at each level */
	public float   percentIncreaseReduction;

	/** Percentage by which score decrease percentage is reduced at each level */
	public float   percentDecreaseReduction;

	/**  Threshold at which score increase is no longer applied */
	public float   percentIncreaseThreshold;

	/** Threshold at which score decrease is no longer applied */
	public float   percentDecreaseThreshold;

	/** Approximate number of nodes added in a non-shift click */
	public int     minChildrenToExpand;

	/** Number of score changes that are saved before the oldest one is undone */
	public int     userHistorySize;

	public FuzzyTreeParameters()
	{
	}

	public FuzzyTreeParameters(  float minExpandPercent, float maxExpandPercent,
								float maxCollapsePercent, float minCollapsePercent,
								float percentIncreaseReduction, float percentIncreaseThreshold,
								float percentDecreaseReduction, float percentDecreaseThreshold,
								int minChildrenToExpand, int userHistorySize)
	{
		this.minExpandPercent = minExpandPercent;
		this.maxExpandPercent = maxExpandPercent;
		this.maxCollapsePercent = maxCollapsePercent;
		this.minCollapsePercent = minCollapsePercent;
		this.percentIncreaseReduction = percentIncreaseReduction;
		this.percentIncreaseThreshold = percentIncreaseThreshold;
		this.percentDecreaseReduction = percentDecreaseReduction;
		this.percentDecreaseThreshold = percentDecreaseThreshold;
		this.minChildrenToExpand = minChildrenToExpand;
		this.userHistorySize = userHistorySize;
	}

}