/** Parameters Dialog.java
 *  Copyright (c) 2001 Donna Malayeri
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package edu.umd.cs.fuzzyTree;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/** Dialog for setting parameters.
 *
 *  @author     Donna Malayeri
 *  @version    24-Jul-01
 */

public class ParametersDialog extends JDialog
{
	GridBagLayout gridBagLayout1 = new GridBagLayout();
	JLabel expandPercentLabel = new JLabel();
	JTextField expandPercentTextField = new JTextField();
	JTextField collapsePercentTextField = new JTextField();
	JLabel collapsePercentLabel = new JLabel();
	JLabel expandReductionLabel = new JLabel();
	JTextField expandReductionTextField = new JTextField();
	JTextField collapseReductionTextField = new JTextField();
	JLabel collapseReductionLabel = new JLabel();
	JLabel expandNumberLabel = new JLabel();
	JTextField expandNumberTextField = new JTextField();
	JPanel buttonPanel = new JPanel();
	JLabel maxExpandPercentLabel = new JLabel();
	JTextField maxExpandPercentTextField = new JTextField();
	JTextField minCollapsePercentTextField = new JTextField();
	JLabel minCollapsePercentLabel = new JLabel();
	JLabel increaseThresholdLabel = new JLabel();
	JTextField increaseThresholdTextField = new JTextField();
	JLabel decreaseThresholdLabel = new JLabel();
	JTextField decreaseThresholdTextField = new JTextField();
	JLabel historySizeLabel = new JLabel();
	JTextField historySizeTextField = new JTextField();
	FlowLayout flowLayout1 = new FlowLayout();

	private FuzzyTreeParameters params;

	public boolean wasCanceled = true;
	JButton okButton = new JButton();
	JButton cancelButton = new JButton();
	JButton helpButton = new JButton();

	// value that is passed in will be modified
	public ParametersDialog(Frame parent, FuzzyTreeParameters p)
	{
		super(parent, "Set Parameters", true);

		try
		{
			jbInit();

			params = p;
			expandPercentTextField.setText(params.minExpandPercent + "");
			maxExpandPercentTextField.setText(params.maxExpandPercent + "");
			collapsePercentTextField.setText(params.maxCollapsePercent + "");
			minCollapsePercentTextField.setText(params.minCollapsePercent + "");
			expandReductionTextField.setText(params.percentIncreaseReduction + "");
			increaseThresholdTextField.setText(params.percentIncreaseThreshold + "");
			collapseReductionTextField.setText(params.percentDecreaseReduction + "");
			decreaseThresholdTextField.setText(params.percentDecreaseThreshold + "");
			expandNumberTextField.setText(params.minChildrenToExpand + "");
			historySizeTextField.setText(params.userHistorySize + "");

			okButton.addActionListener(new OkButtonActionListener());
			cancelButton.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						dispose();
					}
				});

			pack();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	// same object passed in to constructor
	public FuzzyTreeParameters getParameters()
	{
		return params;
	}

	private void jbInit() throws Exception
	{
		expandPercentLabel.setToolTipText("Minimum percentage score increase when expanding");
		expandPercentLabel.setLabelFor(expandPercentTextField);
		expandPercentLabel.setText("Minimum expand percent:");
		this.getContentPane().setLayout(gridBagLayout1);
		expandPercentTextField.setToolTipText("Minimum percentage score increase when expanding");
		expandPercentTextField.setColumns(6);
		collapsePercentTextField.setToolTipText("Maximum percentage score decrease when collapsing");
		collapsePercentTextField.setColumns(6);
		collapsePercentLabel.setText("Maximum collapse percent:");
		collapsePercentLabel.setToolTipText("Maximum percentage score decrease when collapsing");
		expandReductionLabel.setToolTipText("Percentage by which score change percentage is reduced at each level " +
	"when expanding");
		expandReductionLabel.setText("Level percent reduction when expanding:");
		expandReductionTextField.setToolTipText("Percentage by which score change percentage is reduced at each level " +
	"when expanding");
		expandReductionTextField.setColumns(6);
		collapseReductionTextField.setToolTipText("Percentage by which score change percentage is reduced at each level " +
	"when collapsing");
		collapseReductionTextField.setColumns(6);
		collapseReductionLabel.setText("Level percent reduction when collapsing:");
		collapseReductionLabel.setToolTipText("Percentage by which score change percentage is reduced at each level " +
	"when collapsing");
		expandNumberLabel.setText("Children to expand/collapse (normal click):");
		expandNumberLabel.setToolTipText("Approximate number of nodes added/removed in a non-shift click");
		expandNumberTextField.setToolTipText("Approximate number of nodes added/removed in a non-shift click");
		expandNumberTextField.setColumns(3);
		buttonPanel.setLayout(flowLayout1);
		maxExpandPercentLabel.setText("Maximum expand percent:");
		maxExpandPercentLabel.setToolTipText("Maximum percentage score increase when expanding");
		maxExpandPercentTextField.setToolTipText("Maximum percentage score increase when expanding");
		maxExpandPercentTextField.setColumns(6);
		minCollapsePercentTextField.setToolTipText("Minimum percentage score decrease when collapsing");
		minCollapsePercentTextField.setColumns(6);
		minCollapsePercentLabel.setToolTipText("Minimum percentage score decrease when collapsing");
		minCollapsePercentLabel.setText("Minimum collapse percent:");
		increaseThresholdLabel.setToolTipText("Threshold at which score increase is no longer applied");
		increaseThresholdLabel.setText("Increase threshold (percent):");
		increaseThresholdTextField.setToolTipText("Threshold at which score increase is no longer applied");
		increaseThresholdTextField.setColumns(6);
		decreaseThresholdLabel.setToolTipText("Threshold at which score decrease is no longer applied");
		decreaseThresholdLabel.setText("Decrease threshold (percent):");
		decreaseThresholdTextField.setToolTipText("Threshold at which score decrease is no longer applied");
		decreaseThresholdTextField.setColumns(6);
		historySizeLabel.setToolTipText("Number of score changes that are saved before the oldest one is undone");
		historySizeLabel.setText("User history size:");
		historySizeTextField.setToolTipText("Number of score changes that are saved before the oldest one is undone");
		historySizeTextField.setColumns(6);
		this.setModal(true);
		cancelButton.setText("Cancel");
		okButton.setText("OK");
		helpButton.setText("Help");
		helpButton.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				helpButton_actionPerformed(e);
			}
		});
		cancelButton.setText("Cancel");
		okButton.setText("OK");
		this.getContentPane().add(expandPercentTextField, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(collapsePercentTextField, new GridBagConstraints(2, 1, 2, 1, 1.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(collapsePercentLabel, new GridBagConstraints(0, 1, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(expandPercentLabel, new GridBagConstraints(0, 0, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(expandReductionLabel, new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(collapseReductionLabel, new GridBagConstraints(0, 3, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(collapseReductionTextField, new GridBagConstraints(2, 3, 6, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(expandReductionTextField, new GridBagConstraints(2, 2, 3, 1, 1.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(expandNumberLabel, new GridBagConstraints(0, 4, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(expandNumberTextField, new GridBagConstraints(2, 4, 4, 4, 0.0, 1.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(buttonPanel, new GridBagConstraints(0, 6, 10, 1, 1.0, 1.0
			,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL, new Insets(20, 0, 10, 0), 0, 0));
		buttonPanel.add(okButton, null);
		buttonPanel.add(cancelButton, null);
		buttonPanel.add(helpButton, null);
		this.getContentPane().add(increaseThresholdLabel, new GridBagConstraints(5, 2, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(maxExpandPercentTextField, new GridBagConstraints(8, 0, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 10), 0, 0));
		this.getContentPane().add(minCollapsePercentTextField, new GridBagConstraints(7, 1, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(increaseThresholdTextField, new GridBagConstraints(8, 2, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 0), 0, 0));
		this.getContentPane().add(minCollapsePercentLabel, new GridBagConstraints(5, 1, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(maxExpandPercentLabel, new GridBagConstraints(5, 0, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(decreaseThresholdLabel, new GridBagConstraints(5, 3, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(decreaseThresholdTextField, new GridBagConstraints(9, 3, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
			,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 6, 0, 10), 0, 0));
		this.getContentPane().add(historySizeLabel, new GridBagConstraints(0, 5, 2, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 10, 0, 0), 0, 0));
		this.getContentPane().add(historySizeTextField, new GridBagConstraints(2, 5, GridBagConstraints.REMAINDER, 1, 0.0, 0.0
			,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(10, 6, 0, 6), 0, 0));
	}

	private class OkButtonActionListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			try
			{
				params.minExpandPercent = Float.valueOf(expandPercentTextField.getText()).floatValue();
				params.maxExpandPercent = Float.valueOf(maxExpandPercentTextField.getText()).floatValue();
				params.maxCollapsePercent = Float.valueOf(collapsePercentTextField.getText()).floatValue();
				params.minCollapsePercent = Float.valueOf(minCollapsePercentTextField.getText()).floatValue();
				params.percentIncreaseReduction = Float.valueOf(expandReductionTextField.getText()).floatValue();
				params.percentIncreaseThreshold = Float.valueOf(increaseThresholdTextField.getText()).floatValue();
				params.percentDecreaseReduction = Float.valueOf(collapseReductionTextField.getText()).floatValue();
				params.percentDecreaseThreshold = Float.valueOf(decreaseThresholdTextField.getText()).floatValue();
				params.minChildrenToExpand = Integer.parseInt(expandNumberTextField.getText());
				params.userHistorySize = Integer.parseInt(historySizeTextField.getText());

				checkPercent(params.minExpandPercent);

				if (params.maxCollapsePercent < -100f || params.maxCollapsePercent > 0)
					throw new NumberFormatException();
				checkPercent(params.percentIncreaseReduction);
				checkPercent(params.percentDecreaseReduction);

				if (params.minChildrenToExpand < 0 || params.minChildrenToExpand > 20)
					throw new NumberFormatException();

				wasCanceled = false;
				dispose();
			}

			catch (NumberFormatException exception)
			{
				JOptionPane.showMessageDialog(ParametersDialog.this, "Invalid parameter specified");
			}
		}

	}

	private void checkPercent(float value)
	{
		if (value < 0 || value > 100)
			throw new NumberFormatException();
	}

	void helpButton_actionPerformed(ActionEvent e)
	{
		String helpStr = "Minimum/Maximum expand percent: minimum/maximum percentage score increase when expanding\n" +
						 "Minimum/Maximum collapse percent: minimum/maximum percentage score decrease when collapsing\n" +
						 "Level percent reduction when expanding/collapsing:\n" +
						 "      percentage by which score change percentage is reduced at each level\n" +
						 "Increase/Decrease threshold: threshold at which score change is no longer applied\n" +
						 "Children to expand/collapse (normal click):\n" +
						 "      approximate number of nodes added/removed in a non-shift click\n" +
						 "User history size: number of score changes that are saved before the oldest one is undone" ;

		JOptionPane.showMessageDialog(this, helpStr, "Help", JOptionPane.INFORMATION_MESSAGE);
	}
}
